@@ -1,5 +1,5 @@
 //
-// 2021-04-10. Author F. Bertrand <frederic.bertrand@utt.fr>
+// 2021-04-10. Author F. Bertrand <frederic.bertrand@lecnam.net>
 //
 
 #include <stdlib.h> // for NULL
@@ -12,6 +12,7 @@
 
 
 static const R_CallMethodDef callMethods[] = {
+  {"_dgeev_lwork_query_wrapper", (DL_FUNC) &dgeev_lwork_query_wrapper, 3},
   {"_dgemm_wrapper",  (DL_FUNC) &dgemm_wrapper,  17},
   {"_daxpy_wrapper",  (DL_FUNC) &daxpy_wrapper,   5},
   {"_dpotrf_wrapper", (DL_FUNC) &dpotrf_wrapper,  6},
